function getLocalStorage() {
  try {
    return localStorage;
  } catch (err) {
    return null;
  }
}
function getSessionStorage() {
  try {
    return sessionStorage;
  } catch (err) {
    return null;
  }
}
function validateCustomStorage(storage) {
  if (typeof storage.getItem !== 'function' || typeof storage.setItem !== 'function') throw new Error('Persistence: Invalid storage.');
  return storage;
}
function getItem(storage, key, deserialize, callback) {
  var res = storage.getItem(key);
  if (res === null) return;
  if (typeof res !== 'string' && typeof res.then === 'function') res.then(function (val) {
    if (val != null) processValue(deserialize, val, callback);
  });else processValue(deserialize, res, callback);
}
function setItem(storage, key, value, serialize) {
  processValue(serialize, value, function (res) {
    return storage.setItem(key, res);
  });
}
function processValue(func, value, callback) {
  var res = func(value);
  if (res && typeof res.then === 'function') res.then(callback);else callback(res);
}
export default function persistence(key, options) {
  var _options$storage;
  if (options === void 0) {
    options = {};
  }
  if (typeof key !== 'string') throw new Error('Persistence requires a string key.');
  var storage;
  var storageOption = (_options$storage = options.storage) != null ? _options$storage : 'local';
  if (storageOption === 'local') storage = getLocalStorage();else if (storageOption === 'session') storage = getSessionStorage();else storage = validateCustomStorage(storageOption);
  if (!storage) {
    console.warn('Storage unavailable. Persistence disabled.');
    return {};
  }
  return {
    init: function init(origInit, entity) {
      return function () {
        var deserialize = options.deserializeFn || JSON.parse;
        origInit();
        getItem(storage, key, deserialize, entity.set);
      };
    },
    set: function set(origSet, entity) {
      return function () {
        var serialize = options.serializeFn || JSON.stringify;
        origSet.apply(void 0, arguments);
        setItem(storage, key, entity.get(), serialize);
      };
    }
  };
}